import 'package:json_api/src/http/http_message.dart';
import 'package:json_api/src/http/media_type.dart';

/// The response sent by the server and received by the client
class HttpResponse extends HttpMessage {
  HttpResponse(this.statusCode, {String body = ''}) : super(body);

  /// Response status code
  final int statusCode;

  /// True for the requests processed asynchronously.
  /// @see https://jsonapi.org/recommendations/#asynchronous-processing).
  bool get isPending => statusCode == 202;

  /// True for successfully processed requests
  bool get isSuccessful => statusCode >= 200 && statusCode < 300 && !isPending;

  /// True for failed requests (i.e. neither successful nor pending)
  bool get isFailed => !isSuccessful && !isPending;

  /// True for 204 No Content responses
  bool get isNoContent => statusCode == 204;

  bool get hasDocument =>
      body.isNotEmpty &&
      (headers['content-type'] ?? '')
          .toLowerCase()
          .startsWith(MediaType.jsonApi);
}
